@@ -78,22 +78,26 @@ class ASFFile(File):
         'conductor': 'WM/Conductor',
         'remixer': 'WM/ModifiedBy',
         # FIXME engineer
-        'engineer': 'WM/Producer',
+        'producer': 'WM/Producer',
         'grouping': 'WM/ContentGroupDescription',
         'subtitle': 'WM/SubTitle',
-        'album_subtitle': 'WM/SetSubTitle',
+        'discsubtitle': 'WM/SetSubTitle',
         'tracknumber': 'WM/TrackNumber',
         'discnumber': 'WM/PartOfSet',
         # FIXME compilation
         'comment:': 'Description',
         'genre': 'WM/Genre',
         'bpm': 'WM/BeatsPerMinute',
+        'script': 'WM/Script',
+        'language': 'WM/Language',
         'mood': 'WM/Mood',
         'isrc': 'WM/ISRC',
         'copyright': 'Copyright',
         'lyrics': 'WM/Lyrics',
         '~rating': 'WM/SharedUserRating',
-        # FIXME media, catalognumber, barcode
+        'media': 'WM/Media',
+        'barcode': 'WM/Barcode',
+        'catalognumber': 'WM/CatalogNo',
         'label': 'WM/Publisher',
         'encodedby': 'WM/EncodedBy',
         'albumsort': 'WM/AlbumSortOrder',
@@ -106,10 +110,14 @@ class ASFFile(File):
         'musicbrainz_albumartistid': 'MusicBrainz/Album Artist Id',
         'musicbrainz_trmid': 'MusicBrainz/TRM Id',
         'musicbrainz_discid': 'MusicBrainz/Disc Id',
+        'musicbrainz_workid': 'MusicBrainz/Work Id',
+        'musicbrainz_releasegroupid': 'MusicBrainz/Release Group Id',
         'musicip_puid': 'MusicIP/PUID',
         'releasestatus': 'MusicBrainz/Album Status',
         'releasetype': 'MusicBrainz/Album Type',
         'releasecountry': 'MusicBrainz/Album Release Country',
+        'acoustid_id': 'Acoustid/Id',
+        'acoustid_fingerprint': 'Acoustid/Fingerprint',
     }
     __RTRANS = dict([(b, a) for a, b in __TRANS.items()])
 
@@ -147,7 +155,8 @@ class ASFFile(File):
             for mime, data in metadata.images:
                 tag_data = pack_image(mime, data, 3)
                 cover.append(ASFByteArrayAttribute(tag_data))
-            file.tags['WM/Picture'] = cover  
+            if cover:
+                file.tags['WM/Picture'] = cover  
 
         for name, values in metadata.rawitems():
             if name.startswith('lyrics:'):
