@@ -95,6 +95,7 @@ class ID3File(File):
         'TSOP': 'artistsort',
         'TSOT': 'titlesort',
         'TPUB': 'label',
+        'TLAN': 'language',
     }
     __rtranslate = dict([(v, k) for k, v in __translate.iteritems()])
 
@@ -106,8 +107,13 @@ class ID3File(File):
         'MusicBrainz Album Status': 'releasestatus',
         'MusicBrainz TRM Id': 'musicbrainz_trmid',
         'MusicBrainz Disc Id': 'musicbrainz_discid',
+        'MusicBrainz Work Id': 'musicbrainz_workid',
+        'MusicBrainz Release Group Id': 'musicbrainz_releasegroupid',
         'MusicBrainz Album Release Country': 'releasecountry',
         'MusicIP PUID': 'musicip_puid',
+        'Acoustid Fingerprint': 'acoustid_fingerprint',
+        'Acoustid Id': 'acoustid_id',
+        'SCRIPT': 'script',
         'ALBUMARTISTSORT': 'albumartistsort',
         'CATALOGNUMBER': 'catalognumber',
         'BARCODE': 'barcode',
@@ -204,7 +210,7 @@ class ID3File(File):
 
         if settings['clear_existing_tags']:
             tags.clear()
-        if settings['remove_images_from_tags']:
+        if settings['save_images_to_tags'] and metadata.images:
             tags.delall('APIC')
 
         if settings['write_id3v1']:
@@ -264,13 +270,13 @@ class ID3File(File):
                 # Search for an existing POPM frame to get the current playcount
                 for frame in tags.values():
                     if frame.FrameID == 'POPM' and frame.email == settings['rating_user_email']:
-                        count = frame.count
+                        count = getattr(frame, 'count', 0)
                         break
                 else:
                     count = 0
-                
+
                 # Convert rating to range between 0 and 255
-                rating = int(values[0]) * 255 / (settings['rating_steps'] - 1)
+                rating = int(round(float(values[0]) * 255 / (settings['rating_steps'] - 1)))
                 tags.add(id3.POPM(email=settings['rating_user_email'], rating=rating, count=count))
             elif name in self.__rtranslate:
                 frameid = self.__rtranslate[name]
