@@ -27,8 +27,7 @@ import os.path
 import shutil
 import signal
 import sys
-import traceback
-import time
+from collections import deque
 
 # Install gettext "noop" function.
 import __builtin__
@@ -52,8 +51,8 @@ shutil.copystat = _patched_shutil_copystat
 import picard.resources
 import picard.plugins
 
-from picard import musicdns, version_string, log
-from picard.album import Album
+from picard import musicdns, version_string, log, acoustid
+from picard.album import Album, NatAlbum
 from picard.browser.browser import BrowserIntegration
 from picard.browser.filelookup import FileLookup
 from picard.cluster import Cluster, ClusterList, UnmatchedFiles
@@ -62,12 +61,13 @@ from picard.disc import Disc, DiscError
 from picard.file import File
 from picard.formats import open as open_file
 from picard.metadata import Metadata
-from picard.track import Track
+from picard.track import Track, NonAlbumTrack
 from picard.config import IntOption
 from picard.script import ScriptParser
 from picard.ui.mainwindow import MainWindow
 from picard.plugin import PluginManager
 from picard.puidmanager import PUIDManager
+from picard.acoustidmanager import AcoustIDManager
 from picard.util import (
     decode_filename,
     encode_filename,
@@ -81,7 +81,7 @@ from picard.util import (
     partial,
     queue,
     thread,
-    puid_validate
+    mbid_validate
     )
 from picard.webservice import XmlWebService
 
@@ -121,32 +121,23 @@ class Tagger(QtGui.QApplication):
         else:
             userdir = os.environ.get("XDG_CONFIG_HOME", "~/.config")
         self.userdir = os.path.join(os.path.expanduser(userdir), "MusicBrainz", "Picard")
-        self.cachedir = os.path.join(self.userdir, "cache")
 
         # Initialize threading and allocate threads
         self.thread_pool = thread.ThreadPool(self)
-        
+
         self.load_queue = queue.Queue()
-        self.load_queue.run_item = thread.generic_run_item
-
         self.save_queue = queue.Queue()
-        self.save_queue.run_item = thread.generic_run_item
-
         self.analyze_queue = queue.Queue()
-        self.analyze_queue.run_item = analyze_thread_run_item
-        self.analyze_queue.next = self._lookup_puid
-
         self.other_queue = queue.Queue()
-        self.other_queue.run_item = thread.generic_run_item
-        
+
         threads = self.thread_pool.threads
-        threads.append(thread.Thread(self.thread_pool, [self.load_queue,
-                                                        self.other_queue]))
-        threads.append(thread.Thread(self.thread_pool, [self.save_queue]))
-        threads.append(thread.Thread(self.thread_pool, [self.other_queue,
-                                                        self.load_queue]))
-        threads.append(thread.Thread(self.thread_pool, [self.analyze_queue]))
-        
+        threads.append(thread.Thread(self.thread_pool, self.load_queue))
+        threads.append(thread.Thread(self.thread_pool, self.load_queue))
+        threads.append(thread.Thread(self.thread_pool, self.save_queue))
+        threads.append(thread.Thread(self.thread_pool, self.other_queue))
+        threads.append(thread.Thread(self.thread_pool, self.other_queue))
+        threads.append(thread.Thread(self.thread_pool, self.analyze_queue))
+
         self.thread_pool.start()
         self.stopping = False
 
@@ -176,36 +167,41 @@ class Tagger(QtGui.QApplication):
 
         self.setup_gettext(localedir)
 
-        self.xmlws = XmlWebService(self.cachedir)
+        self.xmlws = XmlWebService()
 
         # Initialize fingerprinting
         self._ofa = musicdns.OFA()
         self._ofa.init()
-        self.analyze_queue.ofa = self._ofa
+        self._acoustid = acoustid.AcoustIDClient()
+        self._acoustid.init()
 
         # Load plugins
         self.pluginmanager = PluginManager()
-        user_plugin_dir = os.path.join(self.userdir, "plugins")
-        if not os.path.exists(user_plugin_dir):
-            os.makedirs(user_plugin_dir)
-        self.pluginmanager.load(user_plugin_dir)
+        self.user_plugin_dir = os.path.join(self.userdir, "plugins")
+        if not os.path.exists(self.user_plugin_dir):
+            os.makedirs(self.user_plugin_dir)
+        self.pluginmanager.load_plugindir(self.user_plugin_dir)
         if hasattr(sys, "frozen"):
-            self.pluginmanager.load(os.path.join(os.path.dirname(sys.argv[0]), "plugins"))
+            self.pluginmanager.load_plugindir(os.path.join(os.path.dirname(sys.argv[0]), "plugins"))
         else:
-            self.pluginmanager.load(os.path.join(os.path.dirname(__file__), "plugins"))
+            self.pluginmanager.load_plugindir(os.path.join(os.path.dirname(__file__), "plugins"))
 
         self.puidmanager = PUIDManager()
+        self.acoustidmanager = AcoustIDManager()
 
         self.browser_integration = BrowserIntegration()
 
         self.files = {}
 
         self.clusters = ClusterList()
-        self.albums = []
+        self.albums = {}
+        self.mbid_redirects = {}
 
         self.unmatched_files = UnmatchedFiles()
         self.window = MainWindow()
 
+        self.nats = None
+
     def setup_gettext(self, localedir):
         """Setup locales, load translations, install gettext functions."""
         if self.config.setting["ui_language"]:
@@ -231,13 +227,13 @@ class Tagger(QtGui.QApplication):
             self.log.debug("Loading gettext translation, localedir=%r", localedir)
             self.translation = gettext.translation("picard", localedir)
             self.translation.install(True)
-            ngettext = self.translation.ngettext
+            ungettext = self.translation.ungettext
         except IOError:
             __builtin__.__dict__['_'] = lambda a: a
-            def ngettext(a, b, c):
+            def ungettext(a, b, c):
                 if c == 1: return a
                 else: return b
-        __builtin__.__dict__['ngettext'] = ngettext
+        __builtin__.__dict__['ungettext'] = ungettext
 
     def move_files_to_album(self, files, albumid=None, album=None):
         """Move `files` to tracks on album `albumid`."""
@@ -246,27 +242,41 @@ class Tagger(QtGui.QApplication):
         if album.loaded:
             album.match_files(files)
         else:
-            for file in [file for file in files]:
+            for file in list(files):
                 file.move(album.unmatched_files)
 
-    def move_file_to_album(self, file, albumid=None, album=None):
+    def move_file_to_album(self, file, albumid):
         """Move `file` to a track on album `albumid`."""
         self.move_files_to_album([file], albumid)
 
     def move_file_to_track(self, file, albumid, trackid):
         """Move `file` to track `trackid` on album `albumid`."""
         album = self.load_album(albumid)
-        if album.loaded:
-            album.match_file(file, trackid)
-        else:
-            file.move(album.unmatched_files)
+        file.move(album.unmatched_files)
+        album.run_when_loaded(partial(album.match_file, file, trackid))
+
+    def create_nats(self):
+        if self.nats is None:
+            self.nats = NatAlbum()
+            self.albums["NATS"] = self.nats
+            self.emit(QtCore.SIGNAL("album_added"), self.nats)
+        return self.nats
+
+    def move_file_to_nat(self, file, trackid, node=None):
+        self.create_nats()
+        file.move(self.nats.unmatched_files)
+        nat = self.load_nat(trackid, node=node)
+        nat.run_when_loaded(partial(file.move, nat))
+        if nat.loaded:
+            self.emit(QtCore.SIGNAL("album_updated"), self.nats)
 
     def exit(self):
         self.stopping = True
         self._ofa.done()
+        self._acoustid.done()
         self.thread_pool.stop()
         self.browser_integration.stop()
-        self.xmlws.cleanup()
+        self.xmlws.stop()
 
     def _run_init(self):
         if self._args:
@@ -288,6 +298,16 @@ class Tagger(QtGui.QApplication):
         self.exit()
         return res
 
+    def event(self, event):
+        if event.type() == QtCore.QEvent.FileOpen:
+            f = str(event.file())
+            self.add_files([f])
+            # We should just return True here, except that seems to
+            # cause the event's sender to get a -9874 error, so
+            # apparently there's some magic inside QFileOpenEvent...
+            return 1
+        return QtGui.QApplication.event(self, event)
+
     def _file_loaded(self, result=None, error=None):
         file = result
         if file is not None and error is None and not file.has_error():
@@ -295,11 +315,13 @@ class Tagger(QtGui.QApplication):
             trackid = file.metadata['musicbrainz_trackid']
             albumid = file.metadata['musicbrainz_albumid']
             self.puidmanager.add(puid, trackid)
-            if puid_validate(albumid):
-                if puid_validate(trackid):
-                    self.move_file_to_album(file, albumid)
-                else:
+            if mbid_validate(albumid):
+                if mbid_validate(trackid):
                     self.move_file_to_track(file, albumid, trackid)
+                else:
+                    self.move_file_to_album(file, albumid)
+            elif mbid_validate(trackid):
+                self.move_file_to_nat(file, trackid)
             elif self.config.setting['analyze_new_files']:
                 self.analyze([file])
 
@@ -317,20 +339,19 @@ class Tagger(QtGui.QApplication):
         if new_files:
             self.unmatched_files.add_files(new_files)
             for file in new_files:
-                file.load(self._file_loaded, self.thread_pool)
+                file.load(self._file_loaded)
 
     def process_directory_listing(self, root, queue, result=None, error=None):
-        delay = 10
         try:
             # Read directory listing
             if result is not None and error is None:
                 files = []
-                directories = []
+                directories = deque()
                 try:
                     for path in result:
                         path = os.path.join(root, path)
                         if os.path.isdir(path):
-                            directories.append(path)
+                            directories.appendleft(path)
                         else:
                             try:
                                 files.append(decode_filename(path))
@@ -340,25 +361,22 @@ class Tagger(QtGui.QApplication):
                 finally:
                     if files:
                         self.add_files(files)
-                    delay = min(25 * len(files), 500)
-                    queue = directories + queue
+                    queue.extendleft(directories)
         finally:
             # Scan next directory in the queue
             try:
-                path = queue.pop(0)
+                path = queue.popleft()
             except IndexError: pass
             else:
-                func = partial(self.other_queue.put,
-                               (partial(os.listdir, path),
-                                partial(self.process_directory_listing,
-                                        path, queue),
-                                QtCore.Qt.LowEventPriority))
-                QtCore.QTimer.singleShot(delay, func)
+                self.other_queue.put((
+                    partial(os.listdir, path),
+                    partial(self.process_directory_listing, path, queue),
+                    QtCore.Qt.LowEventPriority))
 
     def add_directory(self, path):
         path = encode_filename(path)
         self.other_queue.put((partial(os.listdir, path),
-                              partial(self.process_directory_listing, path, []),
+                              partial(self.process_directory_listing, path, deque()),
                               QtCore.Qt.LowEventPriority))
 
     def get_file_by_id(self, id):
@@ -417,49 +435,71 @@ class Tagger(QtGui.QApplication):
         """Save the specified objects."""
         files = self.get_files_from_objects(objects, save=True)
         for file in files:
-            file.save(self._file_saved, self.thread_pool,
-                      self.tagger.config.setting)
+            file.save(self._file_saved, self.tagger.config.setting)
 
-    def load_album(self, id, catalognumber=None, discid=None):
-        album = self.get_album_by_id(id)
+    def load_album(self, id, discid=None):
+        id = self.mbid_redirects.get(id, id)
+        album = self.albums.get(id)
         if album:
             return album
-        album = Album(id, catalognumber=catalognumber, discid=discid)
-        self.albums.append(album)
+        album = Album(id, discid=discid)
+        self.albums[id] = album
         self.emit(QtCore.SIGNAL("album_added"), album)
         album.load()
         return album
 
     def reload_album(self, album):
-        album.load(force=True)
+        if album == self.nats:
+            album.update()
+        else:
+            album.load()
 
-    def get_album_by_id(self, id):
-        for album in self.albums:
-            if album.id == id:
-                return album
-        return None
+    def load_nat(self, id, node=None):
+        self.create_nats()
+        nat = self.get_nat_by_id(id)
+        if nat:
+            return nat
+        nat = NonAlbumTrack(id)
+        self.nats.tracks.append(nat)
+        self.nats.update(True)
+        if node:
+            nat._parse_recording(node)
+        else:
+            nat.load()
+        return nat
 
+    def get_nat_by_id(self, id):
+        if self.nats is not None:
+            for nat in self.nats.tracks:
+                if nat.id == id:
+                    return nat
 
     def remove_files(self, files, from_parent=True):
         """Remove files from the tagger."""
         for file in files:
             if self.files.has_key(file.filename):
-                self.analyze_queue.remove(file.filename)
+                file.clear_lookup_task()
+                self._ofa.stop_analyze(file)
+                self._acoustid.stop_analyze(file)
                 del self.files[file.filename]
                 file.remove(from_parent)
 
     def remove_album(self, album):
         """Remove the specified album."""
         self.log.debug("Removing %r", album)
+        album.stop_loading()
         self.remove_files(self.get_files_from_objects([album]))
-        self.albums.remove(album)
+        del self.albums[album.id]
         self.emit(QtCore.SIGNAL("album_removed"), album)
 
     def remove_cluster(self, cluster):
         """Remove the specified cluster."""
         if not cluster.special:
             self.log.debug("Removing %r", cluster)
-            self.remove_files(cluster.files, from_parent=False)
+            files = list(cluster.files)
+            cluster.files = []
+            cluster.clear_lookup_task()
+            self.remove_files(files, from_parent=False)
             self.clusters.remove(cluster)
             self.emit(QtCore.SIGNAL("cluster_removed"), cluster)
 
@@ -511,12 +551,19 @@ class Tagger(QtGui.QApplication):
                 self.window.set_statusbar_message(N_("Could not find PUID for file %s"), file.filename)
                 file.clear_pending()
 
+    @property
+    def use_acoustid(self):
+        return self.config.setting["fingerprinting_system"] == "acoustid"
+
     def analyze(self, objs):
         """Analyze the file(s)."""
         files = self.get_files_from_objects(objs)
         for file in files:
             file.set_pending()
-            self._ofa.analyze(file, partial(self._lookup_puid, file), self.thread_pool)
+            if self.use_acoustid:
+                self._acoustid.analyze(file, partial(file._lookup_finished, 'acoustid'))
+            else:
+                self._ofa.analyze(file, partial(self._lookup_puid, file))
 
     # =======================================================================
     #  Metadata-based lookups
@@ -524,7 +571,7 @@ class Tagger(QtGui.QApplication):
 
     def autotag(self, objects):
         for obj in objects:
-            if isinstance(obj, (File, Cluster)):
+            if isinstance(obj, (File, Cluster)) and not obj.lookup_task:
                 obj.lookup_metadata()
 
     # =======================================================================
@@ -534,21 +581,30 @@ class Tagger(QtGui.QApplication):
     def cluster(self, objs):
         """Group files with similar metadata to 'clusters'."""
         self.log.debug("Clustering %r", objs)
-        if len(objs) <= 1:
-            objs = [self.unmatched_files]
+        if len(objs) <= 1 or self.unmatched_files in objs:
+            files = list(self.unmatched_files.files)
+        else:
+            files = self.get_files_from_objects(objs)
         fcmp = lambda a, b: (
             cmp(a.discnumber, b.discnumber) or
             cmp(a.tracknumber, b.tracknumber) or
             cmp(a.base_filename, b.base_filename))
-        files = self.get_files_from_objects(objs)
         for name, artist, files in Cluster.cluster(files, 1.0):
             QtCore.QCoreApplication.processEvents()
-            cluster = Cluster(name, artist)
-            self.clusters.append(cluster)
-            self.emit(QtCore.SIGNAL("cluster_added"), cluster)
+            cluster = self.load_cluster(name, artist)
             for file in sorted(files, fcmp):
                 file.move(cluster)
 
+    def load_cluster(self, name, artist):
+        for cluster in self.clusters:
+            cm = cluster.metadata
+            if name == cm["album"] and artist == cm["artist"]:
+                return cluster
+        cluster = Cluster(name, artist)
+        self.clusters.append(cluster)
+        self.emit(QtCore.SIGNAL("cluster_added"), cluster)
+        return cluster
+
     # =======================================================================
     #  Utils
     # =======================================================================
@@ -563,9 +619,11 @@ class Tagger(QtGui.QApplication):
         QtGui.QApplication.restoreOverrideCursor()
 
     def refresh(self, objs):
-        albums = [obj for obj in objs if isinstance(obj, Album)]
-        for album in albums:
-            self.reload_album(album)
+        for obj in objs:
+            if isinstance(obj, Album):
+                self.reload_album(obj)
+            elif isinstance(obj, NonAlbumTrack):
+                obj.load()
 
     @classmethod
     def instance(cls):
@@ -577,18 +635,6 @@ class Tagger(QtGui.QApplication):
     def num_pending_files(self):
         return len([file for file in self.files.values() if file.state == File.PENDING])
 
-def analyze_thread_run_item(thread, queue, filename):
-    next = partial(queue.ofa._lookup_fingerprint, queue.next, filename)
-    priority = QtCore.Qt.LowEventPriority + 1
-    try:
-        result = queue.ofa._calculate_fingerprint(filename)
-    except:
-        import traceback
-        thread.log.error(traceback.format_exc())
-        thread.to_main(next, priority, error=sys.exc_info()[1])
-    else:
-        thread.to_main(next, priority, result=result)
-
 def help():
     print """Usage: %s [OPTIONS] [FILE] [FILE] ...
 
