@@ -17,8 +17,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from PyQt4 import QtGui
-from picard.config import FloatOption
+from PyQt4 import QtCore, QtGui
+from picard.config import FloatOption, TextOption
 from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
 from picard.ui.ui_options_matching import Ui_MatchingOptionsPage
 
@@ -37,6 +37,8 @@ class MatchingOptionsPage(OptionsPage):
         FloatOption("setting", "track_matching_threshold", 0.4),
     ]
 
+    _release_type_sliders = {}
+
     def __init__(self, parent=None):
         super(MatchingOptionsPage, self).__init__(parent)
         self.ui = Ui_MatchingOptionsPage()
