@@ -87,12 +87,12 @@ class ScriptingOptionsPage(OptionsPage):
         except OptionsCheckError, e:
             self.ui.script_error.setStyleSheet(self.STYLESHEET_ERROR);
             self.ui.script_error.setText(e.info)
-            return    
+            return
 
     def check(self):
         parser = ScriptParser()
         try:
-            parser.parse(unicode(self.ui.tagger_script.toPlainText()))
+            parser.eval(unicode(self.ui.tagger_script.toPlainText()))
         except Exception, e:
             raise OptionsCheckError(_("Script Error"), str(e))
 
@@ -103,7 +103,7 @@ class ScriptingOptionsPage(OptionsPage):
     def save(self):
         self.config.setting["enable_tagger_script"] = self.ui.enable_tagger_script.isChecked()
         self.config.setting["tagger_script"] = self.ui.tagger_script.toPlainText()
-        
+
     def display_error(self, error):
         pass
 
